@@ -21,34 +21,8 @@ class Bot:
     @staticmethod
     def randomize_between(*responses):
         response = random.choice(responses)
-        if callable(response):
-            return response(self, msg, re.search(regex, msg.data.content))
-            if isinstance(result, str):
-                return result
-            elif isinstance(result, dict):
-                for send, nick in result.items():
-                    self.set_nick(nick)
-                    self.send_msg(send, parent = msg)
-                self.set_nick(self.nick)
-            elif isinstance(result, list):
-                for send in result:
-                    self.send_msg(send, parent = msg)
-        else:
-            if isinstance(response, str):
-                self.send_msg(msgString = result, parent = msg)
-            elif isinstance(response, dict):
-                for send, nick in response.items():
-                    self.set_nick(nick)
-                    self.send_msg(send, parent = msg)
-                self.set_nick(self.nick)
-            elif isinstance(response, list):
-                for send in response:
-                    self.send_msg(send, parent = msg)
-            else:
-                raise BadMessageError(f"Response type: {type(response)}")
-        break
+        return response
 
-    @staticmethod
     def add_command_line_arg(self, *args, **kwargs):
         self.parser.add_argument(*args, **kwargs)
         self.args = self.parser.parse_args()
@@ -57,6 +31,9 @@ class Bot:
         else:
             return self.args
 
+    def send_command(self, type = "send", data = {}):
+        self.conn.send(json.dumps({'type' : type, 'data' : data}))
+
     def __init__(self, nick, **kwargs):
         vars = ["userEmail",
             "userPassword",
@@ -100,14 +77,14 @@ class Bot:
                 self.setattr(item, None)
         self.normowner = re.sub(r"\s+", "", self.owner)
         self.parser = argparse.ArgumentParser(description=f'{nick}: A euphoria.io bot.' if 'description' not in kwargs else kwargs['description'])
-        self.args = self.parser.parse_args()
         if 'customArgs' in kwargs and kwargs['customArgs']:
             self.room = "bots" if "room" not in kwargs else kwargs["room"]
             self.password = "" if "password" not in kwargs else kwargs["password"]
         else:
-            self.add_command_line_arg(args = ["--test", "--debug", "-t"], help = "Used to debug dev builds. Sends bot to &test instead of its default room.", action = 'store_true')
-            self.add_command_line_arg(args = ["--room", "-r"], help = f"Set the room the bot will be placed in. Default: {'bots' if 'room' not in kwargs else kwargs['room']}", default = "bots" if "room" not in kwargs else kwargs["room"])
-            self.add_command_line_arg(args = ["--password", "-p"], help = "Set the password for the room the bot will be placed in.", default = "" if "password" not in kwargs else kwargs["password"])
+            self.add_command_line_arg("--test", "--debug", "-t", help = "Used to debug dev builds. Sends bot to &test instead of its default room.", action = 'store_true')
+            self.add_command_line_arg("--room", "-r", help = f"Set the room the bot will be placed in. Default: {'bots' if 'room' not in kwargs else kwargs['room']}", default = "bots" if "room" not in kwargs else kwargs["room"])
+            self.add_command_line_arg("--password", "-p", help = "Set the password for the room the bot will be placed in.", default = "" if "password" not in kwargs else kwargs["password"])
+            self.args = self.parser.parse_args()
             self.room = self.args.room if self.args.test != True or _copy else "test"
             self.password = self.args.password
             if not self._copy:
@@ -134,11 +111,11 @@ class Bot:
     def send_msg(self, msgString, parent = None):
         try:
             if isinstance(parent, AttrDict):
-                self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString, 'parent': parent.data.id}}))
+                self.send_command(data = {'content': msgString, 'parent': parent.data.id})
             elif isinstance(parent, str):
-                self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString, 'parent': parent}}))
+                self.send_command(data = {'content': msgString, 'parent': parent})
             elif parent is None:
-                self.conn.send(json.dumps({'type': 'send', 'data': {'content': msgString}}))
+                self.send_command(data = {'content': msgString})
             else:
                 raise BadMessageError(f'message not sent, because type of parent was {type(parent)}. \nParent printed: \n{parent}')
             reply = None
@@ -172,49 +149,37 @@ class Bot:
         self.start()
 
     def start(self, function = None, advanced = False):
-        if not (self.advanced or advanced):
-            try:
-                for msgJSON in self.conn:
-                    msg = AttrDict(json.loads(msgJSON))
-                    if msg.type == 'ping-event':
-                        self._handle_ping(msg)
-                    elif msg.type == 'send-event' and msg.data.sender.name != self.nick:
-                        self._handle_message(msg)
-                    elif msg.type == 'error':
-                        log(msg, error = True, _copy = self._copy)
-                    elif msg.type == 'bounce-event':
-                        self._handle_auth(msg)
-                    else:
-                        self._handle_other(msg)
-            except KilledError:
-                pass
+        if not advanced and not self.advanced:
+            self.set_handler('ping-event', self._handle_ping)
+            self.set_handler('send-event', self._handle_message)
+            self.set_handler('bounce-event', self._handle_auth)
         else:
             if self.function is None and function is not None and callable(function):
                 self.function = function
             else:
-                log("No function passed to advanced bot's start function. Using filler function", _copy = self._copy)
+                log("No valid function passed to advanced bot's start function. Using filler function", _copy = self._copy)
                 self.function = lambda msg: None
-            try:
-                for msgJSON in self.conn:
-                    msg = AttrDict(json.loads(msgJSON))
-                    if msg.type == 'send-event' and msg.data.sender.name != self.nick:
-                        if re.search(f'^!kill @{re.escape(self.normname)}$', msg.data.content) is not None:
-                            self._handle_kill(msg)
-                        if re.search(f'^!forcekill @{re.escape(self.normname)}$', msg.data.content) is not None:
-                            self.kill()
-                        if re.search(f'^!restart @{re.escape(self.normname)}$', msg.data.content) is not None:
-                            self.restart(msg = msg)
-                    if msg.type in self.handlers:
-                        self.handlers[msg.type](msg)
+        try:
+            for msgJSON in self.conn:
+                msg = AttrDict(json.loads(msgJSON))
+                if msg.type == 'send-event':
+                    if re.search(f'^!kill @{re.escape(self.normname)}$', msg.data.content) is not None:
+                        self._handle_kill(msg)
+                    if re.search(f'^!forcekill @{re.escape(self.normname)}$', msg.data.content) is not None:
+                        self.kill()
+                    if re.search(f'^!restart @{re.escape(self.normname)}$', msg.data.content) is not None:
+                        self.restart(msg = msg)
+                if msg.type == 'error':
+                    log(msg, error = True, _copy = self._copy)
+                elif msg.type in self.handlers:
+                    self.handlers[msg.type](msg)
+                else:
+                    if callable(self.function):
+                        self.function(msg)
                     else:
-                        if function is not None and callable(function):
-                            function(msg)
-                        elif callable(self.function):
-                            self.function(msg)
-                        else:
-                            log("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True, _copy = self._copy)
-            except KilledError:
-                pass
+                        log("Advanced start must be given a callable message handler function that takes an AttrDict as its argument.", error = True, _copy = self._copy)
+        except KilledError:
+            pass
 
     def _is_privileged(self, user):
         if "is_manager" in user or user.name == self.owner:
@@ -243,7 +208,7 @@ class Bot:
                 self.killed = "/me has died."
 
     def _handle_ping(self, msg):
-        self.conn.send(json.dumps({'type': 'ping-reply', 'data': {'time': msg.data.time}}))
+        self.send_command(type = 'ping-reply', data = {'time': msg.data.time})
 
     def _handle_message(self, msg):
         if re.search(f'^!kill @{re.escape(self.normname)}$', msg.data.content) is not None:
@@ -300,7 +265,7 @@ class Bot:
             self.send_msg(f"Bot not killed because you are not a host or @{self.normowner}, and this bot is marked as important.\nFor emergencies, use !forcekill.", parent = msg)
 
     def _handle_auth(self, pw):
-        self.conn.send(json.dumps({'type': 'auth', 'data': {'type': 'passcode', 'passcode': pw}}))
+        self.send_command(type = 'auth', data = {'type': 'passcode', 'passcode': pw})
         reply = None
         i = 0
         for msgJSON in self.conn:
@@ -323,14 +288,14 @@ class Bot:
             self.handlers[msg.type](msg)
 
     def set_nick(self, nick):
-        self.conn.send(json.dumps({'type': 'nick', 'data': {'name': nick}}))
+        self.send_command(type = 'nick', data = {'name': nick})
 
     def kill(self):
         self.conn.close()
         raise KilledError(f'{self.normname} killed.')
 
     def get_userlist(self):
-        self.conn.send(json.dumps({'type': 'who'}))
+        self.send_command(type = 'who')
         reply = None
         i = 0
         for msgJSON in self.conn:
@@ -389,7 +354,7 @@ class Bot:
 
     def login(self, email = None, password = None, setDefaults = False):
         if email is not None and password is not None:
-            self.conn.send(json.dumps({'type': 'login', 'data': {'namespace' : "email", 'id' : email, 'password' : password}}))
+            self.send_command(type = 'login', data = {'namespace' : "email", 'id' : email, 'password' : password}})
             if setDefaults:
                 self.userEmail = email
                 self.userPassword = password
@@ -422,7 +387,7 @@ class Bot:
         copy.conn.close()
         del copy.conn
         copy.connect()
-        copy.conn.send(json.dumps({'type': 'pm-initiate', 'data': {'user_id' : id}}))
+        self.send_command(type = 'pm-initiate', data = {'user_id' : id})
         to = None
         i = 0
         for msgJSON in copy.conn:
